package meerkat.pollingstation;

/**
 * Created by Arbel on 5/31/2016.
 */

import com.google.common.util.concurrent.FutureCallback;
import meerkat.protobuf.Comm;
import meerkat.protobuf.PollingStation;

import javax.annotation.PostConstruct;
import javax.servlet.ServletContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import java.io.IOException;
import java.util.Iterator;

import static meerkat.pollingstation.PollingStationConstants.POLLING_STATION_WEB_SCANNER_ERROR_PATH;
import static meerkat.pollingstation.PollingStationConstants.POLLING_STATION_WEB_SCANNER_SCAN_PATH;
import static meerkat.rest.Constants.MEDIATYPE_PROTOBUF;

/**
 * Implements a Web-App interface for {@link meerkat.pollingstation.PollingStationScanner.Producer}
 * This class depends on {@link meerkat.pollingstation.PollingStationWebScanner} and works in conjunction with it
 */
@Path("/")
public class PollingStationScannerWebApp implements PollingStationScanner.Producer {

    @Context
    ServletContext servletContext;

    Iterator<FutureCallback<PollingStation.ScannedData>> callbacks;

    /**
     * This method is called by the Jetty engine when instantiating the servlet
     */
    @PostConstruct
    public void init() throws Exception{
        callbacks = ((PollingStationWebScanner.CallbackAccessor) servletContext.getAttribute(PollingStationWebScanner.CALLBACKS_ATTRIBUTE_NAME)).getCallbackIterator();
    }

    @POST
    @Path(POLLING_STATION_WEB_SCANNER_SCAN_PATH)
    @Consumes(MEDIATYPE_PROTOBUF)
    @Produces(MEDIATYPE_PROTOBUF)
    @Override
    public Comm.BoolMsg newScan(PollingStation.ScannedData scannedData) {

        boolean handled = false;

        while (callbacks.hasNext()){

            callbacks.next().onSuccess(scannedData);
            handled = true;

        }

        return Comm.BoolMsg.newBuilder()
                .setValue(handled)
                .build();

    }

    @POST
    @Path(POLLING_STATION_WEB_SCANNER_ERROR_PATH)
    @Consumes(MEDIATYPE_PROTOBUF)
    @Produces(MEDIATYPE_PROTOBUF)
    @Override
    public Comm.BoolMsg reportScanError(PollingStation.ErrorMsg errorMsg) {

        boolean handled = false;

        while (callbacks.hasNext()){

            callbacks.next().onFailure(new IOException(errorMsg.getMsg()));
            handled = true;

        }

        return Comm.BoolMsg.newBuilder()
                .setValue(handled)
                .build();

    }

}
