package meerkat.pollingstation;

import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import com.google.common.util.concurrent.FutureCallback;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.*;

import meerkat.protobuf.PollingStation.*;
import org.glassfish.jersey.servlet.ServletContainer;
import org.glassfish.jersey.server.ResourceConfig;

import meerkat.protobuf.PollingStation.ScannedData;
import meerkat.rest.*;

/**
 * Created by Arbel on 05/05/2016.
 */

public class PollingStationWebScanner implements PollingStationScanner.Consumer{

    public final static String CALLBACKS_ATTRIBUTE_NAME = "callbacks";

    private final Server server;
    private final List<FutureCallback<ScannedData>> callbacks;

    public PollingStationWebScanner(HTTPAddress address) {

        callbacks = new LinkedList<>();

        server = new Server(address.getPort());

        ServletContextHandler servletContextHandler = new ServletContextHandler(server, address.getAddress());
        servletContextHandler.setAttribute(CALLBACKS_ATTRIBUTE_NAME, new CallbackAccessor());

        ResourceConfig resourceConfig = new ResourceConfig(PollingStationScannerWebApp.class);
        resourceConfig.register(ProtobufMessageBodyReader.class);
        resourceConfig.register(ProtobufMessageBodyWriter.class);

        ServletHolder servletHolder = new ServletHolder(new ServletContainer(resourceConfig));

        servletContextHandler.addServlet(servletHolder, "/*");
    }

    @Override
    public void start() throws Exception {
        server.start();
    }

    @Override
    public void stop() throws Exception {
        server.stop();
    }

    @Override
    public void subscribe(FutureCallback<ScannedData> scanCallback) {
        callbacks.add(scanCallback);
    }

    public class CallbackAccessor {
        public Iterator<FutureCallback<ScannedData>> getCallbackIterator() {
            return callbacks.iterator();
        }
    }

}
